@@ -76,7 +76,7 @@ int     resolve_local(const char *addr)
     char   *dest;
     const char *bare_dest;
     struct addrinfo *res0 = 0;
-    int     len;
+    ssize_t len;
 
 #define RETURN(x) \
     do { \
@@ -170,6 +170,7 @@ int     main(int argc, char **argv)
     mail_conf_read();
     vstream_printf("%s\n", resolve_local(argv[1]) ? "yes" : "no");
     vstream_fflush(VSTREAM_OUT);
+    return (0);
 }
 
 #endif
